
import java.io.*;
 
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class Mileage extends JFrame implements ActionListener , WindowListener {

	private static Mileage instance = null;
	private final String DEFAULT_FILENAME = "records.jmd";
	private String serialize_file = "records.jmd";
	
	private String windowTitle = "Mileage";
	private JScrollPane tableScroller;
	private JPanel tableFrame;
	private JPanel buttonFrame;
	private JTable infoTable;
	private JMenuBar menubar;
	
	private boolean modified = false;
	private boolean initialLoading = true;
	
	private Color bgcolor = Color.WHITE;
	
	// Action commands
	private final String ADD_RECORD = "Add Record";
	private final String REMOVE_RECORD = "Remove Record";
	private final String CREATE_REPORT = "Create Report";
	private final String OPEN = "Open...";
	private final String SAVE = "Save";
	private final String SAVE_AS = "Save As...";
	private final String QUIT = "Quit";
	
	private Vector vehicles = new Vector();
	private Vector dates = new Vector();
	private Vector odometer = new Vector();
	private Vector gallons = new Vector();
	private Vector costs = new Vector();

	public static Mileage getInstance() {
		if( instance == null ) instance = new Mileage();
		return instance;
	}

	protected Mileage() { 
		super();
		readData();
		makeGUI(); }
	
	private void makeGUI() {
		setTitle( windowTitle );
		setBackground( bgcolor );
		setLocation( 100 , 100 );
		getContentPane().setLayout( new BorderLayout() );
		setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );
		setSize( new Dimension( 400 , 300 ) );
		setResizable( false );
		addWindowListener( this );
		buildMenus();
		menubar.setBackground( bgcolor );
		setJMenuBar( menubar );
		tableFrame = new JPanel();
		tableFrame.setBackground( bgcolor );
		addTable();
		tableFrame.add( tableScroller );
		getContentPane().add( tableFrame , BorderLayout.CENTER );
		
		buttonFrame = new JPanel();
		addButtons();
		getContentPane().add( buttonFrame , BorderLayout.SOUTH );
		
		setVisible( true );
	}
	
	private void buildMenus() {
		JMenu file = new JMenu( "File" );
		file.setBackground( bgcolor );
		menubar = new JMenuBar();
		file.add( makeMenuItem( OPEN ) );
		file.add( makeMenuItem( SAVE ) );
		file.add( makeMenuItem( SAVE_AS ) );
		file.add( makeMenuItem( QUIT ) );
		menubar.add( file );
	}
		
	private JMenuItem makeMenuItem( String text ) {
		JMenuItem item = new JMenuItem( text );
		item.setBackground( bgcolor );
		item.addActionListener( this );
		return item;
	}
		
		
	private void addTable() {
		
				
		final Vector vehicleCol = vehicles;
		final Vector dateCol = dates;
		final Vector odometerCol = odometer;
		final Vector gallonsCol = gallons;
		final Vector costCol = costs;
		
		
		TableModel model = new AbstractTableModel() {
			public int getColumnCount() {
				return 5;
			}
			public int getRowCount() {
				return vehicleCol.size();
			}
			public String getColumnName( int column ) {
				switch( column ) {
				case 0: return "Vehicle";
				case 1: return "Date";
				case 2: return "Odometer";
				case 3: return "Gallons";
				case 4: return "Cost";
				default: return "";
				}
			}
			public Object getValueAt( int row, int col ) {
				switch( col ) {
				case 0: return (String)vehicleCol.get( row );
				case 1: return (String)dateCol.get( row );
				case 2: return (String)odometerCol.get( row ) + " m.";
				case 3: return (String)gallonsCol.get( row ) + " gal.";
				case 4: return "$" + (String)costCol.get( row );
				default: return null;
				}
			}
		};
		
		infoTable = new JTable( vehicleCol.size() , 5 );
		infoTable.setBackground( bgcolor );
		infoTable.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		infoTable.setModel( model );
		infoTable.setAutoResizeMode( JTable.AUTO_RESIZE_ALL_COLUMNS );
		tableScroller = new JScrollPane( infoTable );
		tableScroller.setPreferredSize( new Dimension( 380 , 280 ) );
		tableScroller.setBackground( bgcolor );
		tableScroller.setVerticalScrollBar( new JScrollBar() );
		
		
	}
	
	private void addButtons() {
		buttonFrame.setLayout( new FlowLayout() );
		buttonFrame.add( makeButton( ADD_RECORD ) );
		buttonFrame.add( makeButton( REMOVE_RECORD ) );
		buttonFrame.add( makeButton( CREATE_REPORT ) );
		buttonFrame.setBackground( bgcolor );
	}
	
	private JButton makeButton( String text ) {
		JButton button = new JButton( text );
		button.setActionCommand( text );
		button.setToolTipText( text );
		button.addActionListener( this );
		return button;
	}
	
	
	public void actionPerformed( ActionEvent e ) {
		String command = e.getActionCommand();
		
		if( command.equals( ADD_RECORD ) ) {
			RecordWindow.getInstance();
		} else if( command.equals( REMOVE_RECORD ) ) {
			if( dates.size() > 0 ) {
				RemoveRecordWindow.getInstance( infoTable.getSelectedRow() );
			} else {
				JOptionPane.showMessageDialog( null , 
						"There are no records to remove." , 
						"Oops" , JOptionPane.WARNING_MESSAGE );
			}
		} else if( command.equals( CREATE_REPORT ) ) {
			switch( dates.size() ) {
				case 0:
					JOptionPane.showMessageDialog( null , 
								"There are no records to report on." , 
								"Oops" , JOptionPane.WARNING_MESSAGE );
					break;
				case 1: 
					ReportWindow.getInstance( 0 );
					break;
				default:
					CheckGasWindow.getInstance();
			}
		
		} else if( command.equals( QUIT ) ) {
			windowClosing( null );
		} else if( command.equals( SAVE ) ) {
			saveData( serialize_file );
			modified = false;
		} else if( command.equals( SAVE_AS ) ) {
			// open file menu, then back up serialize_file
			JFileChooser dialog = new JFileChooser( "." );
			dialog.setMultiSelectionEnabled( false );
			dialog.setFileFilter( new MileageFileFilter() );
			dialog.showSaveDialog( null );
			String filePath = null;
			try {
				filePath = dialog.getSelectedFile().toString();
			} catch( Exception ex ){}
			if( filePath != null )
				saveData( dialog.getSelectedFile().toString() );
		} else if( command.equals( OPEN ) ) {
			// open file menu, then back up serialize_file
			JFileChooser dialog = new JFileChooser( "." );
			dialog.setMultiSelectionEnabled( false );
			dialog.setFileFilter( new MileageFileFilter() );
			dialog.showOpenDialog( null );
			String filePath = null;
			try {
				filePath = dialog.getSelectedFile().toString();
			} catch( Exception ex ){}
			if( filePath != null ) {
				serialize_file = filePath;
				readData();
				updateTable();
			}
		}
	}
	
	public void addRecord( String[] data ) {
		vehicles.add( data[0] );
		dates.add( data[1] );
		odometer.add( data[2] );
		gallons.add( data[3] );
		costs.add( data[4] );
		updateTable();
		modified = true;
	}
	
	public void removeRecord( String date ) {
		int index = dates.indexOf( date );
		vehicles.remove( index );
		dates.remove( index );
		odometer.remove( index );
		gallons.remove( index );
		costs.remove( index );
		updateTable();
		modified = true;
	}
	
	public Vector getVehicles() { return vehicles; }
	public Vector getDates() { return dates; }
	public Vector getOdometer() { return odometer; }
	public Vector getGallons() { return gallons; }
	public Vector getCosts() { return costs; }
	
	public void updateTable() {
		tableFrame.setVisible( false );
		tableFrame.removeAll();
		tableFrame.setBackground( bgcolor );
		addTable();
		tableFrame.add( tableScroller );
		getContentPane().add( tableFrame , BorderLayout.CENTER );
		tableFrame.setVisible( true );
	}
	
	private void readData() {
		try {
			FileInputStream ins = new FileInputStream( serialize_file );
			ObjectInputStream p = new ObjectInputStream( ins );
			vehicles = (Vector)p.readObject();
			dates = (Vector)p.readObject();
			odometer = (Vector)p.readObject();
			gallons = (Vector)p.readObject();
			costs = (Vector)p.readObject();
			ins.close();
			modified = false;
			
		} catch ( Exception e ) {
			if( !initialLoading ) {
				JOptionPane.showMessageDialog( null , 
					"An error occurred while trying to read " + serialize_file + "!" , 
					"Oops" , JOptionPane.ERROR_MESSAGE );
			}
		} finally {
			initialLoading = false;
		}
	}
	
	private void saveData( String fileName ) {
		//serialize the map
		try {
			FileOutputStream out = new FileOutputStream( fileName );
			ObjectOutputStream s = new ObjectOutputStream( out );
			s.writeObject( vehicles );
			s.writeObject( dates );
			s.writeObject( odometer );
			s.writeObject( gallons );
			s.writeObject( costs );
			s.flush();
			out.close();
		} catch( Exception ex ) {
			JOptionPane.showMessageDialog( null , 
				"An error occurred while trying to write " + fileName + "!" , 
				"Oops" , JOptionPane.ERROR_MESSAGE );
		}
	}
	
	public void windowClosing( WindowEvent e ) {
		if( modified ) {
			int sel = JOptionPane.showConfirmDialog( null , "Save changes to " + serialize_file + "?" , "Quitting..." , 
					JOptionPane.YES_NO_CANCEL_OPTION , JOptionPane.QUESTION_MESSAGE );
			switch( sel ) {
				case JOptionPane.YES_OPTION:
					saveData( serialize_file );
				case JOptionPane.NO_OPTION:
					dispose();
					System.exit( 0 );
					break;
			}
		} else {
			dispose();
			System.exit( 0 );
		}
	}
	
	public void windowActivated( WindowEvent e ) {}
	public void windowClosed( WindowEvent e ) {}
	public void windowDeactivated( WindowEvent e ) {}
	public void windowDeiconified( WindowEvent e ) {}
	public void windowIconified( WindowEvent e ) {}
	public void windowOpened( WindowEvent e ) {}
	
	public static void main( String[] args ) {
		Mileage.getInstance();
	}
	
}